'use server'

import Docker from 'dockerode';

// 只在服务器端执行的操作
const docker = new Docker({ socketPath: '/var/run/docker.sock' });

export async function createContainers(count: number, baseName: string, startingPorts: [number, number]) {
    const [vncStart, sshStart] = startingPorts;
    const promises = [];

    // 只在服务器端加载 cpu-features
    if (typeof window === 'undefined') {
        try {
            const cpuFeatures = require('cpu-features');  // 仅在服务器端加载
            console.log('CPU Features:', cpuFeatures); // 打印所有的 CPU 特性对象
        } catch (error) {
            console.error('Error loading cpu-features:', error);
        }
    }

    for (let i = 0; i < count; i++) {
        const containerName = `${baseName}-${i}`;
        const vncPort = vncStart + i;
        const sshPort = sshStart + i;

        try {
            const container = await docker.createContainer({
                Image: 'dockerp.com/dcsunset/ubuntu-vnc:latest', // 使用你的镜像
                name: containerName,
                ExposedPorts: {
                    '5900/tcp': {}, // VNC 默认端口
                    '22/tcp': {},   // SSH/sSFTP 默认端口
                },
                HostConfig: {
                    PortBindings: {
                        '5900/tcp': [{ HostPort: vncPort.toString() }],
                        '22/tcp': [{ HostPort: sshPort.toString() }],
                    },
                },
            });

            promises.push(container);
        } catch (error) {
            console.error('Error creating container:', error);
            throw new Error(`Failed to create container ${containerName}`);
        }
    }

    return Promise.all(promises);
}

// 其他容器操作如 stop、remove 等
export async function stopContainer(containerName: string) {
    try {
        const container = docker.getContainer(containerName);
        await container.stop();
        return { message: `Container ${containerName} stopped successfully.` };
    } catch (error) {
        console.error('Error stopping container:', error);
        throw new Error(`Failed to stop container ${containerName}`);
    }
}

export async function removeContainer(containerName: string) {
    try {
        const container = docker.getContainer(containerName);
        await container.remove();
        return { message: `Container ${containerName} removed successfully.` };
    } catch (error) {
        console.error('Error removing container:', error);
        throw new Error(`Failed to remove container ${containerName}`);
    }
}

export async function getContainerInfo(containerName: string) {
    try {
        const container = docker.getContainer(containerName);
        const info = await container.inspect();
        return {
            containerName: info.Name,
            state: info.State.Status,
            ports: info.NetworkSettings.Ports,
        };
    } catch (error) {
        console.error('Error retrieving container info:', error);
        throw new Error(`Failed to retrieve info for container ${containerName}`);
    }
}

export async function startContainer(containerName: string) {
    try {
        const container = docker.getContainer(containerName);
        await container.start();
        return { message: `Container ${containerName} started successfully.` };
    } catch (error) {
        console.error('Error starting container:', error);
        throw new Error(`Failed to start container ${containerName}`);
    }
}
